<?php

namespace LiamW\PostMacros\Admin\Controller;

use LiamW\PostMacros\Entity\Macro;
use XF\Admin\Controller\AbstractController;
use XF\Mvc\ParameterBag;

class Macros extends AbstractController
{
	public function actionIndex()
	{
		$page = $this->filterPage();
		$perPage = 50;

		$finder = $this->finder('LiamW\PostMacros:Macro')->where('type', 'global')->orderTitle();
		$finder->limitByPage($page, $perPage);
		$filter = $this->filter('_xfFilter', [
			'text' => 'str',
			'prefix' => 'bool'
		]);
		if (strlen($filter['text']))
		{
			$finder->searchTitle($filter['text'], $filter['prefix']);
		}

		$total = $finder->total();
		$macros = $finder->fetch();

		$viewParams = [
			'macros' => $macros,

			'filter' => $filter,

			'page' => $page,
			'perPage' => $perPage,
			'total' => $total
		];

		return $this->view('LiamW\PostMacros:Macros\Index', 'liamw_postmacros_macros_list', $viewParams);
	}

	public function actionAdd()
	{
		return $this->macroAddEdit($this->getMacroRepo()->setupBaseGlobalMacro());
	}

	public function actionEdit(ParameterBag $params)
	{
		$macro = $this->assertMacroExists($params['macro_id']);

		return $this->macroAddEdit($macro);
	}

	protected function macroAddEdit(Macro $macro)
	{
		$viewParams = [
			'macro' => $macro
		];

		return $this->view('LiamW\PostMacros:Macros\Edit', 'liamw_postmacros_edit', $viewParams);
	}

	protected function macroSaveProcess(Macro $macro)
	{
		$form = $this->formAction();

		$input = $this->filter([
			'title' => 'str',
			'auto_submit' => 'bool',
			'metadata' => $this->getMacroRepo()->getMetaDataFilters()
		]);
        $type = $this->filter('change_category', 'array');
        $input['change_category'] = $this->getMacroRepo()->getChangeTypeBySave($type, $macro);
		$input['content'] = $this->plugin('XF:Editor')->fromInput('content');

		$form->basicEntitySave($macro, $input);

		return $form;
	}

	public function actionSave(ParameterBag $params)
	{
		$this->assertPostOnly();

		if ($params->macro_id)
		{
			$macro = $this->assertMacroExists($params->macro_id);
		}
		else
		{
			$macro = null;
		}

		$macro = $this->getMacroRepo()->setupBaseGlobalMacro($macro);
		$this->macroSaveProcess($macro)->run();

		return $this->redirect($this->buildLink('post-macros'));
	}

	public function actionDelete(ParameterBag $params)
	{
		$macro = $this->assertMacroExists($params['macro_id']);

		return $this->plugin('XF:Delete')->actionDelete(
			$macro,
			$this->buildLink('post-macros/delete', $macro),
			$this->buildLink('post-macros/edit', $macro),
			$this->buildLink('post-macros'),
			$macro->title
		);
	}

	public function actionPermissions(ParameterBag $params)
	{
		return $this->getGlobalMacroPermissionPlugin()->actionList($params);
	}

	public function actionPermissionsEdit(ParameterBag $params)
	{
		return $this->getGlobalMacroPermissionPlugin()->actionEdit($params);
	}

	public function actionPermissionsSave(ParameterBag $params)
	{
		return $this->getGlobalMacroPermissionPlugin()->actionSave($params);
	}

	/**
	 * @param $macroId
	 *
	 * @return \LiamW\PostMacros\Entity\Macro
	 */
	protected function assertMacroExists($macroId)
	{
		return $this->assertRecordExists('LiamW\PostMacros:Macro', $macroId);
	}

	/**
	 * @return \LiamW\PostMacros\Repository\Macro
	 */
	protected function getMacroRepo()
	{
		return $this->repository('LiamW\PostMacros:Macro');
	}

	protected function getGlobalMacroPermissionPlugin()
	{
		/** @var \LiamW\PostMacros\ControllerPlugin\GlobalMacroPermission $plugin */
		$plugin = $this->plugin('LiamW\PostMacros:GlobalMacroPermission');

		return $plugin;
	}
}