<?php

namespace LiamW\PostMacros\ChangeCategory;

use LiamW\PostMacros\Entity\Macro;
use XF\Mvc\Entity\Entity;
use XF\Mvc\View;

/**
 * Class AbstractHandler
 * @package LiamW\PostMacros\ChangeCategory
 */
abstract class AbstractHandler
{
    /**
     * @var null
     */
	protected $shortName = null;

    /**
     * @var string
     */
    protected $contentType;

    /**
     * @var string
     */
    protected $pattern = '/\{\+\?id+(?P<id>.+)\}/';

    /**
     * @var string
     */
    protected $patternBase = '/\{\+\?id+(.*?)\}/';

    /**
     * @var string
     */
    protected $message;

    /**
     * @var null
     */
    protected $categoryId = null;

    /**
     * AbstractHandler constructor.
     * @param $message
     */
	public function __construct($message)
	{
		if (!$this->shortName)
		{
			throw new \BadMethodCallException("Change category implementations must override the shortName property");
		}
		$this->message = $message;
	}

    /**
     * @return mixed
     */
	abstract protected function setCategory();

    /**
     * @return mixed
     */
	abstract public function label();

    /**
     * @return mixed
     */
	public function getCategory()
    {
        return $this->setCategory();
    }

    /**
     * @return string
     */
	public function getContentType()
    {
        return $this->contentType;
    }

    /**
     * @return string
     */
    public function getMessage()
    {
        return $this->message;
    }

    /**
     * @return bool|mixed|null
     */
    public function getIdCategory()
    {
        preg_match($this->pattern, $this->message, $matches);
        if(!empty($matches))
        {
            $this->categoryId = $matches['id'];
            return $this->categoryId;
        }
        return false;
    }

    /**
     * @return string|string[]|null
     */
    public function removeCategoryId()
    {
        return preg_replace($this->patternBase, '', $this->message);
    }

    /**
     * @return mixed
     */
    abstract public function testIfCategoryExist();

    /**
     * @param $shortName
     * @return \XF\Mvc\Entity\Finder
     */
	protected function finder($shortName)
    {
        return \XF::em()->getFinder($shortName);
    }

    /**
     * @return bool
     */
    public function canChangeCategory()
    {
        if(!\XF::visitor()->hasPermission('liamw_postmacros', 'can_change_category'))
        {
            return false;
        }
        return true;
    }
}