<?php

namespace LiamW\PostMacros\Job;

use XF\Job\AbstractJob;

class EnrollmentRebuild extends AbstractJob
{
	protected $defaultData = [
		'steps' => 0,
		'permission_combination_id' => 0,
		'position' => [0, 0]
	];

	public function run($maxRunTime)
	{
		$start = microtime(true);

		$this->data['steps']++;

		$macrosEnrolmentFinder = $this->app->finder('LiamW\PostMacros:GlobalMacroEnrolment')->with('User')
			->with('Macro')
			->order('user_id')
			->order('macro_id');

		if ($this->data['permission_combination_id'])
		{
			$macrosEnrolmentFinder->where('User.permission_combination_id', $this->data['permission_combination_id']);
		}

		$macrosEnrolmentFinder
			->where('user_id', '>=', $this->data['position'][0])
			->where('macro_id', '>', $this->data['position'][1]);

		$macroEnrolments = $macrosEnrolmentFinder->fetch(50);
		if (!$macroEnrolments->count())
		{
			return $this->complete();
		}

		/** @var \LiamW\PostMacros\Entity\GlobalMacroEnrolment $macroEnrolment */
		foreach ($macroEnrolments AS $macroEnrolment)
		{
			if (microtime(true) - $start >= $maxRunTime)
			{
				break;
			}

			$this->data['position'] = [$macroEnrolment->user_id, $macroEnrolment->macro_id];

			\XF::asVisitor($macroEnrolment->User, function() use ($macroEnrolment)
			{
				if (!$macroEnrolment->Macro->canEnroll($error, true))
				{
					$macroEnrolment->delete();
				}
			});
		}

		return $this->resume();
	}

	public function getStatusMessage()
	{
		$actionPhrase = \XF::phrase('rebuilding');
		$typePhrase = \XF::phrase('liamw_postmacros_global_macro_enrollment');

		return sprintf('%s... %s %s', $actionPhrase, $typePhrase, str_repeat(' . ', $this->data['steps']));
	}

	public function canCancel()
	{
		return false;
	}

	public function canTriggerByChoice()
	{
		return false;
	}
}