<?php

namespace LiamW\PostMacros;

use XF\Data\Editor;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Manager;
use XF\Mvc\Entity\Structure;
use XF\Template\Templater;

class Listener
{
	public static function editorRender(Templater $templater, &$type, &$template, array &$params)
	{
		$params['postMacros'] = [];

		if (in_array('lwPostMacrosInsert', $params['removeButtons']))
		{
			return;
		}
		if (!isset($templater->pageParams['forum']) || $templater->pageParams['forum']->liamw_postmacros_allow_macros)
		{
			$params['postMacros'] = \XF::repository('LiamW\PostMacros:Macro')->getMacrosForEditor();

			if (empty($params['postMacros']))
			{
				$params['removeButtons'][] = 'lwPostMacrosInsert';
			}
		}
		else
		{
			$params['removeButtons'][] = 'lwPostMacrosInsert';
		}
	}

	public static function editorButtonData(array &$buttons, Editor $editorData)
	{
		$buttons['lwPostMacrosInsert'] = [
			'fa' => 'fa-bolt',
			'title' => \XF::phrase('liamw_postmacros_insert_macro'),
			'type' => 'dropdown'
		];
	}

	public static function forumEntityStructure(Manager $em, Structure &$structure)
	{
		$structure->columns['liamw_postmacros_allow_macros'] = ['type' => Entity::BOOL, 'default' => 1];
	}

	public static function userOptionEntityStructure(Manager $em, Structure &$structure)
	{
		$structure->columns['liamw_postmacros_expanded_macros'] = ['type' => Entity::BOOL, 'default' => 1];
	}

	public static function userChange(\XF\Service\User\ContentChange $changeService, array &$updates)
	{
		$updates['xf_liamw_post_macros'] = ['user_id', 'emptyable' => false];
		$updates['xf_liamw_post_macros_global_enrolment'] = ['user_id', 'emptyable' => false];
		$updates['xf_liamw_post_macros_shared'] = ['user_id', 'emptyable' => false];
	}

	public static function userDeleteClean(\XF\Service\User\DeleteCleanUp $deleteService, array &$deletes)
	{
		// Note: Macros are handled using a custom step (to ensure dependencies are removed)

		$deletes['xf_liamw_post_macros_shared'] = 'user_id = ?';
		$deletes['xf_liamw_post_macros_global_enrolment'] = 'user_id = ?';
	}
}