<?php

namespace LiamW\PostMacros\MetaData\Group;

use LiamW\PostMacros\MetaData\Handler\AbstractHandler;
use XF\Mvc\Controller;
use XF\Mvc\Entity\Entity;

abstract class AbstractGroup
{
	protected $contentType;

	/**
	 * @var AbstractHandler[]|null
	 */
	protected $handlerCache = null;

	/**
	 * AbstractGroup constructor.
	 *
	 * @param string $contentType
	 */
	public function __construct($contentType)
	{
		$this->contentType = $contentType;
	}

	public function getGroupLabel()
	{
		return \XF::phrase('liamw_postmacros_x_metadata', ['type' => \XF::app()
			->getContentTypePhrase($this->getContentType())]);
	}

	public function processMetadata(Entity $entity, Controller $controller)
	{
		$metadata = $controller->filter([
			'macro_metadata' => $this->getHandlerInputTypes()
		]);

		foreach ($metadata['macro_metadata'] AS $handler => $value)
		{
			if ($value !== null)
			{
				$this->getMetaDataHandler($handler)->process($entity, $value);
			}
		}

		$entity->save();
	}

	protected function getHandlerInputTypes()
	{
		$filterTypes = [];

		foreach ($this->getMetaDataHandlers() AS $handler)
		{
			$filterTypes[$handler->getShortName()] = $handler->getFilterType();
		}

		return $filterTypes;
	}

	/**
	 * @return string
	 */
	public function getContentType()
	{
		return $this->contentType;
	}

	/**
	 * @return AbstractHandler[]
	 * @throws \Exception
	 */
	public function getMetaDataHandlers()
	{
		if ($this->handlerCache == null)
		{
			$this->handlerCache = [];

			foreach ($this->getMetaDataHandlerClasses() AS $handlerClass)
			{
				$this->handlerCache[$handlerClass] = AbstractHandler::create($handlerClass, $this);
			}
		}

		return $this->handlerCache;
	}

	/**
	 * @param $shortName
	 *
	 * @return AbstractHandler
	 * @throws \Exception
	 */
	public function getMetaDataHandler($shortName)
	{
		if (!isset($this->handlerCache[$shortName]))
		{
			if (!in_array($shortName, $this->getMetaDataHandlerClasses()))
			{
				throw new \InvalidArgumentException("Cannot to get grouped handler of different group.");
			}

			$this->handlerCache[$shortName] = AbstractHandler::create($shortName, $this);
		}

		return $this->handlerCache[$shortName];
	}

	abstract public function getMetaDataHandlerClasses();
}