<?php

namespace LiamW\PostMacros\MetaData\Handler;

use XF\Entity\Thread;
use LiamW\PostMacros\Entity\Macro;
use XF\Mvc\Entity\Entity;
use XF\Mvc\View;

class ThreadControls extends AbstractHandler
{
	protected $shortName = 'LiamW\PostMacros:ThreadControls';

	protected function renderMetaDataInput(View $view, $inputName)
	{
		return $view->renderTemplate('public:liamw_postmacros_metadata_template_thread_actions', [
			'value' => $this->value,
			'inputName' => $inputName
		]);
	}

	protected function renderMetaData(View $view)
	{
		return $view->renderTemplate('public:liamw_postmacros_metadata_view_thread_actions', [
			'value' => $this->value
		]);
	}

	public function getFilterType()
	{
		return [
			'close_thread' => '?bool',
			'stick_thread' => '?bool'
		];
	}

	public function getMetaDataGroup()
	{
		return 'thread';
	}

	protected function verifyMetadataValue(Macro $macro, &$newValue)
	{
		return true;
	}

	protected function processInternal(Entity $entity, $value)
	{
		/** @var Thread $entity */
		// Never unlock a locked thread
		if ($entity->discussion_open && $entity->canLockUnlock())
		{
			$entity->discussion_open = isset($value['close_thread']) && $value['close_thread'] ? 0 : 1;
		}

		// Never unsticky a stickied thread
		if (!$entity->sticky && $entity->canStickUnstick())
		{
			$entity->sticky = isset($value['stick_thread']) && $value['stick_thread'];
		}
	}
}