<?php

namespace LiamW\PostMacros\Permission;

use XF\Entity\PermissionCombination;
use XF\Permission\FlatContentPermissions;

class GlobalMacros extends FlatContentPermissions
{
	protected function getContentType()
	{
		return 'liamw_macros_globalmacro';
	}

	public function getContentList()
	{
		return \XF::app()->finder('LiamW\PostMacros:Macro')->globalMacros()->fetch();
	}

	protected function getFinalPerms($contentId, array $calculated, array &$childPerms)
	{
		return $this->builder->finalizePermissionValues($calculated['liamw_postmacros_global']);
	}

	public function getAnalysisTypeTitle()
	{
		return \XF::phrase('liamw_postmacros_global_macro_permissions');
	}

	protected function getFinalAnalysisPerms($contentId, array $calculated, array &$childPerms)
	{
		return $this->builder->finalizePermissionValues($calculated);
	}

	public function rebuildCombination(PermissionCombination $combination, array $basePerms)
	{
		parent::rebuildCombination($combination, $basePerms);

		// This job de-enrolls users from macros they no longer have permission to enroll onto
		\XF::app()->jobManager()
			->enqueueUnique('liamw_postmacros_EnrollmentRebuild_pc' . $combination->permission_combination_id, 'LiamW\PostMacros:EnrollmentRebuild', ['permission_combination_id' => $combination->permission_combination_id]);
	}

	public function isValidPermission(\XF\Entity\Permission $permission)
	{
		return $permission->permission_id == 'enroll' && $permission->permission_group_id == 'liamw_postmacros_global';
	}
}