<?php

namespace LiamW\PostMacros\Repository;

use LiamW\PostMacros\ChangeCategory\AbstractHandler;
use LiamW\PostMacros\MetaData\Handler\AbstractHandler as MetaDataHandler;
use LiamW\PostMacros\MetaData\Group\AbstractGroup as MetaDataGroupHandler;
use LiamW\PostMacros\PostMacros;
use LiamW\PostMacros\XF\Entity\User;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\ArrayCollection;
use XF\Mvc\Entity\Repository;
use XF\Mvc\ParameterBag;
use XF\PrintableException;

class Macro extends Repository
{
	protected static $metadataGroupHandlerCache = null;
	protected static $metadataHandlerCache = null;

	public function setupBaseMacro(\LiamW\PostMacros\Entity\Macro $macro = null)
	{
		if (!$macro)
		{
			$macro = $this->em->create('LiamW\PostMacros:Macro');
			$macro->user_id = \XF::visitor()->user_id;
		}

		return $macro;
	}

	public function setupBaseGlobalMacro(\LiamW\PostMacros\Entity\Macro $macro = null)
	{
		if (!$macro)
		{
			$macro = $this->em->create('LiamW\PostMacros:Macro');
			$macro->user_id = null;
			$macro->type = 'global';
		}

		return $macro;
	}

	/**
	 * @param string $error
	 *
	 * @return bool
	 * @deprecated This method is now deprecated, and will be removed in a future version.
	 * @see User::canCreateMacro()
	 *
	 */
	public function canCreateMacro(&$error = null)
	{
		return PostMacros::visitor()->canCreateMacro($error);
	}

	protected function findMacrosForEditor()
	{
		$userId = \XF::visitor()->user_id;

		return $this->finder('LiamW\PostMacros:Macro')->with("Sharing|$userId")->with("Enrolment|$userId")
			->whereOr(['user_id', $userId], ["Sharing|$userId.enrolled", 1], ["Enrolment|$userId.user_id", '!=', null])
			->orderTitle();
	}

	public function getMacrosForEditor()
	{
		$user = \XF::visitor();

		if (!$user->hasPermission('liamw_postmacros', 'use'))
		{
			return [];
		}

		$macros = $this->findMacrosForEditor()->fetch();

		$output = [];

		foreach ($macros as $macro)
		{

			$output[] = [
				'macro_id' => $macro->macro_id,
				'title' => $macro->title,
				'hasTokens' => count($macro->custom_tokens) > 0 || $macro->getCategory($this->app()->request()->getRequestUri())
			];
		}

		return $output;
	}

	public function getMacrosForList()
	{
		$user = \XF::visitor();
		$userId = $user->user_id;

		/*$macros = $this->finder('LiamW\PostMacros:Macro')->with("Sharing|$userId")->with("Enrolment|$userId")
			->whereOr(['user_id', $userId], ["Sharing|$userId.enrolled", 1], ["Enrolment|$userId.user_id", '!=', null])
			->order('title')->fetch();


		$unEnrolledShared = $this->finder('LiamW\PostMacros:Macro')->with("Sharing|$userId")
			->where('user_id', '!=', $userId)->where('type', 'user')
			->where("Sharing|$userId.enrolled", 0)->fetch()->filter(function (\LiamW\PostMacros\Entity\Macro $macro) {
				return $macro->canEnroll();
			});

		$unEnrolledGlobal = $this->finder('LiamW\PostMacros:Macro')->with("Enrolment|$userId")
			->with('Permissions|' . $user->permission_combination_id)->where('type', 'global')
			->where("Enrolment|$userId.user_id", null)->fetch()
			->filter(function (\LiamW\PostMacros\Entity\Macro $macro) {
				return $macro->canEnroll();
			});*/

		$viewableMacros = $this->finder('LiamW\PostMacros:Macro')
			->with("list")
			->whereOr(['user_id', $userId], ["Sharing|$userId.user_id", "!=", null], ['type', 'global'])
			->fetch();

		$macros = $viewableMacros->filter(function(\LiamW\PostMacros\Entity\Macro $macro)
		{
			return ($macro->type == 'user' && ($macro->user_id == \XF::visitor()->user_id || $macro->Sharing[\XF::visitor()->user_id]->enrolled)) || $macro->Enrolment[\XF::visitor()->user_id];
		});

		$unEnrolledShared = $viewableMacros->filter(function(\LiamW\PostMacros\Entity\Macro $macro)
		{
			return $macro->type == 'user' && $macro->user_id != \XF::visitor()->user_id && !$macro->Sharing[\XF::visitor()->user_id]->enrolled;
		});

		$unEnrolledGlobal = $viewableMacros->filter(function(\LiamW\PostMacros\Entity\Macro $macro)
		{
			return $macro->type == 'global' && !$macro->Enrolment[\XF::visitor()->user_id] && $macro->canEnroll();
		});

		return [$macros, $unEnrolledShared, $unEnrolledGlobal];
	}

	/**
	 * @return \LiamW\PostMacros\Finder\Macro|\XF\Mvc\Entity\Finder
	 */
	public function findActiveMacrosForList()
	{
		$userId = \XF::visitor()->user_id;

		return $this->finder('LiamW\PostMacros:Macro')->with('list')->whereOr(['user_id', $userId], ["Sharing|$userId.enrolled", 1], ["Enrolment|$userId.user_id", '!=', null])->order('display_order')->orderTitle();
	}

	/**
	 * @return \LiamW\PostMacros\Finder\Macro|\XF\Mvc\Entity\Finder
	 */
	public function findInactiveMacroForList()
	{
		$userId = \XF::visitor()->user_id;

		$finder = $this->finder('LiamW\PostMacros:Macro')->with('list')->orderTitle();
		$finder->whereOr([['type', 'global'], ["Enrolment|$userId.user_id", null]], ["Sharing|$userId.enrolled", 0]);

		return $finder;
	}

	/**
	 * @return \LiamW\PostMacros\MetaData\Group\AbstractGroup[]
	 */
	public function getMetaDataGroupHandlers()
	{
		if (self::$metadataGroupHandlerCache === null)
		{
			$classes = $this->app()->getContentTypeField('liamw_postmacros_metadata_group_handler');

			foreach ($classes AS $contentType => $metadataClass)
			{
				$metadataClass = \XF::extendClass($metadataClass);

				self::$metadataGroupHandlerCache[$contentType] = new $metadataClass($contentType);
			}
		}

		return self::$metadataGroupHandlerCache;
	}

    /**
     * @param $shortName
     * @param $message
     * @return AbstractHandler|mixed
     * @throws \Exception
     */
    public function getChangeTypeHandler($shortName, $message)
    {
        $className = \XF::stringToClass($shortName, '%s\ChangeCategory\%s');
        $className = \XF::extendClass($className);

        if (!class_exists($className))
        {
            throw new \InvalidArgumentException("Invalid Change category class ($shortName) specified.");
        }

        $obj = new $className($message);

        if ($obj instanceof AbstractHandler)
        {
            return $obj;
        }

        throw new \InvalidArgumentException("Invalid Change category class ($shortName) specified.");
    }

	/**
	 * @param $contentType
	 *
	 * @return \LiamW\PostMacros\MetaData\Group\AbstractGroup
	 */
	public function getMetaDataGroupHandler($contentType)
	{
		if (self::$metadataGroupHandlerCache === null)
		{
			$this->getMetaDataGroupHandlers();
		}

		return self::$metadataGroupHandlerCache[$contentType];
	}

	public function getMetaDataHandlersGrouped(array $values = [])
	{
		if (self::$metadataHandlerCache === null)
		{
			$groupHandlers = $this->getMetaDataGroupHandlers();

			foreach ($groupHandlers as $contentType => $groupHandler)
			{
				/** @var MetaDataGroupHandler $groupHandler */
				foreach ($groupHandler->getMetaDataHandlerClasses() AS $handlerShortName)
				{
					self::$metadataHandlerCache[$contentType][] = MetaDataHandler::create($handlerShortName, $groupHandler, isset($values[$handlerShortName]) ? $values[$handlerShortName] : null);
				}
			}
		}

		return self::$metadataHandlerCache;
	}

	public function getMetaDataFilters()
	{
		$metadataHandlersGrouped = $this->getMetaDataHandlersGrouped();

		$return = [];

		foreach ($metadataHandlersGrouped AS $metadataHandlers)
		{
			foreach ($metadataHandlers AS $metadataHandler)
			{
				/** @var MetaDataHandler $metadataHandler */
				$return[$metadataHandler->getShortName()] = $metadataHandler->getFilterType();
			}
		}

		return $return;
	}

	public final function replaceCustomPlaceholders(\LiamW\PostMacros\Entity\Macro $macro, array $placeholderValues)
	{
		$matchCount = 0;
        $exists = [];
        $valueExists = [];
		return preg_replace_callback('/\{\?\+(?P<required>!?)(?P<label>.+)\}(\s)?/U', function($match) use ($placeholderValues, &$matchCount, &$exists, &$valueExists)
		{
			$endCharacter = isset($match[3]) ? $match[3] : '';
			$keyValueExists  = preg_replace('#\s#', '_', trim($match['label']));
            $keyValueExists  = strtolower($keyValueExists);
            if(!in_array(trim($match['label']), $exists))
            {
                $exists[] = trim($match['label']);
                $valueExists[$keyValueExists] = $placeholderValues[$matchCount] ? $placeholderValues[$matchCount] . $endCharacter : '';
                $matchCount++;
            }
			return $valueExists[$keyValueExists];
		}, $macro->content);
	}

	public final function renderMacroPlaceholders($macroContent, $contextUrl)
	{
		$placeholderPairs = $this->getMacroPlaceholderPairs($this->getParamsFromUrl($contextUrl));

		return strtr($macroContent, $placeholderPairs);
	}

	/**
	 * Get placeholder=>value pairs.
	 *
	 * @param \XF\Mvc\ParameterBag $params Route parameters from the editor form action.
	 *
	 * @return array
	 */
	protected final function getMacroPlaceholderPairs(ParameterBag $params)
	{
		$placeholderRoots = $this->getPlaceholderRoots($params);
		$adminSpecifiedPlaceholders = preg_split('/\r?\n/', trim(\XF::options()->liamw_postmacros_placeholders), -1, PREG_SPLIT_NO_EMPTY);

		$output = [];

		foreach ($adminSpecifiedPlaceholders AS $adminSpecifiedPlaceholder)
		{
			list($placeholder, $field) = explode('=>', $adminSpecifiedPlaceholder, 2);

			if (!preg_match('/^(?P<root>[a-z_]+)\.(?P<parts>(?:\.?[A-Z]?[0-9a-z_])+)$/', $field, $matches) || isset($output[$placeholder]))
			{
				if (\XF::$debugMode)
				{
					$val = "[color=red]**[i]{$placeholder}[/i]: Placeholder field not valid format**[/color]";
				}
				else
				{
					continue;
				}
			}
			else
			{
				$root = $matches['root'];
				$parts = explode('.', $matches['parts']);

				if (!array_key_exists($root, $placeholderRoots))
				{
					if (\XF::$debugMode)
					{
						$val = "[color=red]**[i]{$placeholder}[/i]: Placeholder root not valid (this might be expected)**[/color]";
					}
					else
					{
						continue;
					}
				}
				else
				{
					try
					{
						$val = $placeholderRoots[$root];

						foreach ($parts AS $part)
						{
							$val = $val->__get($part);
						}

					} catch (\Throwable $e)
					{
						if (\XF::$debugMode)
						{
							$val = "[color=red]**[i]{$placeholder}[/i]: Placeholder field not valid: {$e->getMessage()}**[/color]";
						}
						else
						{
							continue;
						}
					}
				}
			}

			if (!is_string($val) && !is_int($val) && !is_float($val))
			{
				if (\XF::$debugMode)
				{
					$val = "[color=red]**[i]{$placeholder}[/i]: Placeholder value not valid (might be null)**[/color]";
				}
				else
				{
					continue;
				}
			}
			else if (empty($val) && \XF::$debugMode)
			{
				$val = "[color=rgb(250, 197, 28)]**[i]{$placeholder}[/i]: Placeholder value is empty**[/color]";
			}

			$output["{{$placeholder}}"] = $val;
		}

		$output = array_merge($output, $this->getAdditionalMacroPlaceholderPairs($params));

		return $output;
	}

	public function getMacroPlaceholders()
	{
		$adminPlaceholders = [];

		$placeholders = preg_split('/\r?\n/', trim(\XF::options()->liamw_postmacros_placeholders), -1, PREG_SPLIT_NO_EMPTY);

		foreach ($placeholders AS $placeholder)
		{
			list($adminPlaceholders[]) = explode('=>', $placeholder);
		}

		return array_map(function($placeholder)
		{
			return "{{$placeholder}}";
		}, $adminPlaceholders);
	}
    public function getChangeTypeBySave($types, $macro)
    {
        $output = [];
        if(!empty($types))
        {
            foreach ($types as $type)
            {
                $output[$type] = $macro->list_change_category[$type]['handler'];
            }
            return $output;
        }
        return [];
    }
	/**
	 * Extension point for add-ons.
	 *
	 * @param \XF\Mvc\ParameterBag $params
	 *
	 * @return array
	 */
	protected function getAdditionalMacroPlaceholderPairs(ParameterBag $params)
	{
		return [];
	}

	protected final function getPlaceholderRoots(ParameterBag $params)
	{
		$roots = [
			'visitor' => \XF::visitor()
		];

		if ($forum = $this->em->find('XF:Forum', $params->node_id))
		{
			$roots['forum'] = $forum;
		}

		if ($thread = $this->em->find('XF:Thread', $params->thread_id))
		{
			$roots['thread'] = $thread;
			$roots['forum'] = $thread->Forum;
		}

		if ($conversation = $this->em->find('XF:ConversationMaster', $params->conversation_id))
		{
			$roots['conversation'] = $conversation;
		}

		if (isset($this->app()->registry()['addOns']['XFRM']) && $resourceCategory = $this->em->find('XFRM:Category', $params->resource_category_id))
		{
			$roots['resource_category'] = $resourceCategory;
		}

		if (isset($this->app()->registry()['addOns']['XFRM']) && $resource = $this->em->find('XFRM:ResourceItem', $params->resource_id))
		{
			$roots['resource'] = $resource;
			$roots['resource_category'] = $resource->Category;
		}

		\XF::fire('liamw_postmacros_placeholder_roots', [$params, &$roots]);

		return $roots;
	}

	protected function getParamsFromUrl($url)
	{
		$routePath = $this->app()->request()->getRoutePathFromUrl($url);
		$routeMatch = $this->app()->router('public')->routeToController($routePath);

		return $routeMatch->getParameterBag();
	}

	public function addSharingsToMacros(AbstractCollection $macros)
	{
		/** @var \LiamW\PostMacros\Entity\Macro[] $macros */

		$macroIds = $macros->keys();

		$sharings = $this->finder('LiamW\PostMacros:MacroSharing')->where('macro_id', $macroIds)->fetch()->groupBy('macro_id', 'user_id');

		foreach ($macros AS $macro)
		{
			if (empty($sharings[$macro->macro_id]))
			{
				$macros[$macro->macro_id]->hydrateRelation('Sharing', new ArrayCollection([]));
			}
			else
			{
				$macros[$macro->macro_id]->hydrateRelation('Sharing', new ArrayCollection($sharings[$macro->macro_id]));
			}
		}
	}
}