<?php

namespace LiamW\PostMacros;

use LiamW\PostMacros\Entity\Macro;
use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;
use XF\Entity\CodeEvent;
use XF\Mvc\Entity\AbstractCollection;
use XF\PrintableException;

class Setup extends AbstractSetup
{
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;

	public function checkRequirements(&$errors = [], &$warnings = [])
	{
		if ($this->addOn && $this->addOn->version_id < 4030270)
		{
			$warnings[] = "It is recommended you upgrade to Post Macros 5 from version 4.3.2. Please restore your XF1 backup and upgrade Post Macros from the GitHub repository (<a href=\"https://github.com/lw-addons/xenforo_post-macros/releases\"></a>). Data may be lost if you choose to continue.";
		}
	}

	public function installStep1()
	{
		$this->createTable('xf_liamw_post_macros', function (Create $table)
		{
			$table->comment("Added by Post Macros");
			$table->addColumn('macro_id', 'int')->autoIncrement();
			$table->addColumn('user_id', 'int')->nullable(true);
			$table->addColumn('type', 'enum')->values(['user', 'global'])->setDefault('user');
			$table->addColumn('title', 'varchar', 150);
			$table->addColumn('content', 'text');
			$table->addColumn('auto_submit', 'bool')->setDefault(0);
			$table->addColumn('display_order', 'int')->setDefault(0);
            $table->addColumn('change_category', 'mediumblob');
			$table->addColumn('metadata', 'mediumblob');
			$table->addKey('user_id');
		});

		$this->createTable('xf_liamw_post_macros_shared', function (Create $table)
		{
			$table->comment("Added by Post Macros");
			$table->addColumn('macro_id', 'int');
			$table->addColumn('user_id', 'int');
			$table->addColumn('can_write', 'tinyint');
			$table->addColumn('enrolled', 'bool');
			$table->addPrimaryKey(['macro_id', 'user_id']);
		});

		$this->createTable('xf_liamw_post_macros_global_enrolment', function (Create $table)
		{
			$table->comment("Added by Post Macros");
			$table->addColumn('macro_id', 'int');
			$table->addColumn('user_id', 'int');
			$table->addPrimaryKey(['macro_id', 'user_id']);
		});

		$this->alterTable('xf_forum', function (Alter $table)
		{
			$table->addColumn('liamw_postmacros_allow_macros', 'bool')->setDefault(1);
		});

		$this->alterTable('xf_user_option', function (Alter $table)
		{
			$table->addColumn('liamw_postmacros_expanded_macros', 'bool')->setDefault(0)->after('email_on_conversation');
		});
	}

	public function upgrade5000011Step1()
	{
		$this->createTable('xf_liamw_post_macros', function (Create $table)
		{
			$table->comment("Added by Post Macros");
			$table->addColumn('macro_id', 'int')->autoIncrement();
			$table->addColumn('user_id', 'int')->nullable(true);
			$table->addColumn('type', 'enum')->values(['user', 'global'])->setDefault('user');
			$table->addColumn('title', 'varchar', 50);
			$table->addColumn('content', 'text');
			$table->addColumn('metadata', 'mediumblob');
			$table->addKey('user_id');
		});

		$this->createTable('xf_liamw_post_macros_shared', function (Create $table)
		{
			$table->comment("Added by Post Macros");
			$table->addColumn('macro_id', 'int');
			$table->addColumn('user_id', 'int');
			$table->addColumn('can_write', 'tinyint');
			$table->addPrimaryKey(['macro_id', 'user_id']);
		});

		$this->createTable('xf_liamw_post_macros_global_enrolment', function (Create $table)
		{
			$table->comment("Added by Post Macros");
			$table->addColumn('macro_id', 'int');
			$table->addColumn('user_id', 'int');
			$table->addPrimaryKey(['macro_id', 'user_id']);
		});

		$this->alterTable('xf_forum', function (Alter $table)
		{
			$table->addColumn('liamw_postmacros_allow_macros', 'bool')->setDefault(0)->comment("Added by Post Macros");
		});
	}

	public function upgrade5000011Step2()
	{
		$macros = $this->db()->fetchAllKeyed("
			SELECT * FROM liam_post_macros
		", 'macro_id');

		$db = $this->db();
		$db->beginTransaction();

		foreach ($macros AS $macro)
		{
			$db->insert('xf_liamw_post_macros', [
				'user_id' => $macro['user_id'],
				'type' => 'user',
				'title' => $macro['title'],
				'content' => $macro['content'],
				'metadata' => json_encode([
					'LiamW\PostMacros:ThreadTitle' => $macro['thread_title'],
					'LiamW\PostMacros:ThreadControls' => ['close_thread' => $macro['lock_thread'] ? 1 : 0]
				])
			]);
		}

		$db->commit();
	}

	public function upgrade5000011Step3()
	{
		$adminMacros = $this->db()->fetchAllKeyed("
			SELECT * FROM liam_post_macros_admin
		", 'macro_id');

		$db = $this->db();
		$db->beginTransaction();

		foreach ($adminMacros AS $macro)
		{
			$db->insert('xf_liamw_post_macros', [
				'user_id' => 0,
				'type' => 'global',
				'title' => $macro['title'],
				'content' => $macro['content'],
				'metadata' => json_encode([
					'LiamW\PostMacros:ThreadTitle' => $macro['thread_title'],
					'LiamW\PostMacros:ThreadControls' => ['close_thread' => $macro['lock_thread'] ? 1 : 0]
				])
			]);
		}

		$db->commit();
	}

	public function upgrade5000011Step4()
	{
		$this->dropTable('liam_post_macros');
		$this->dropTable('liam_post_macros_admin');

		$this->alterTable('xf_user_option', function (Alter $table)
		{
			$table->dropColumns([
				'post_macros_hide_new_thread_reply',
				'post_macros_hide_thread_quick_reply',
				'post_macros_hide_new_conversation_reply',
				'post_macros_hide_conversation_quick_reply',
				'post_macros_hide_other'
			]);
		});

		$this->alterTable('xf_forum', function (Alter $table)
		{
			$table->dropColumns('post_macros_enable');
		});
	}

	public function upgrade5000014Step1()
	{
		$this->alterTable('xf_liamw_post_macros_shared', function (Alter $table)
		{
			$table->addColumn('enrolled', 'bool');
		});
	}

	public function upgrade5010010Step1()
	{
		$this->alterTable('xf_liamw_post_macros', function (Alter $table)
		{
			$table->addColumn('auto_submit', 'bool')->setDefault(0)->after('content');
			$table->addKey('user_id');
		});
	}

	public function upgrade5010010Step2()
	{
		$this->alterTable('xf_liamw_post_macros_global_enrolment', function (Alter $table)
		{
			$table->dropIndexes('user_id');
		});
	}

	public function upgrade5010270Step1()
	{
		$this->alterTable('xf_liamw_post_macros', function (Alter $table)
		{
			$table->changeColumn('title', 'varchar', 150);
			$table->addColumn('display_order', 'int')->setDefault(0)->after('auto_submit');
		});
	}

	public function upgrade5010270Step2()
	{
		$this->alterTable('xf_user_option', function (Alter $table)
		{
			$table->addColumn('liamw_postmacros_expanded_macros', 'bool')->setDefault(0)->after('email_on_conversation');
		});
	}

	public function upgrade5010271Step1(array $stepState)
	{
		$stepState = array_merge([
			'position' => 0
		], $stepState);

		/** @var AbstractCollection|Macro[] $macros */
		$macros = \XF::finder('LiamW\PostMacros:Macro')->where('macro_id', '>', $stepState['position'])->fetch(50);

		if (!$macros->count())
		{
			return null;
		}

		\XF::db()->beginTransaction();

		foreach ($macros AS $macro)
		{
			$stepState['position'] = $macro->macro_id;

			$metadata = $macro->metadata;

			if (!empty($metadata['LiamW\PostMacros:ThreadActions']))
			{
				$threadControlMetadata = $metadata['LiamW\PostMacros:ThreadActions'];
				$metadata['LiamW\PostMacros:ThreadControls'] = $threadControlMetadata;
			}

			unset($metadata['LiamW\PostMacros:ThreadActions']);

			$macro->fastUpdate('metadata', $metadata);
		}

		\XF::db()->commit();

		return $stepState;
	}
    public function upgrade5010610Step1()
    {
        $this->alterTable('xf_liamw_post_macros', function (Alter $table)
        {
            $table->addColumn('change_category', 'mediumblob');
        });
    }
	public function uninstallStep1()
	{
		$this->dropTable('xf_liamw_post_macros');
		$this->dropTable('xf_liamw_post_macros_shared');
		$this->dropTable('xf_liamw_post_macros_global_enrolment');

		$this->alterTable('xf_forum', function (Alter $table)
		{
			$table->dropColumns(['liamw_postmacros_allow_macros']);
		});
	}
}