<?php

namespace LiamW\PostMacros\XF\Service\User;

class DeleteCleanUp extends XFCP_DeleteCleanUp
{
	public function __construct(\XF\App $app, $userId, $userName)
	{
		parent::__construct($app, $userId, $userName);

		array_splice($this->steps, 3, 0, "stepDeletePostMacros");
	}

	protected function stepDeletePostMacros($lastOffset, $maxRunTime)
	{
		// This is essentially copied verbatim from the \XF\Service\User\DeleteCleanUp::stepDeleteProfilePosts
		// method changing only the finder to the macros finder.

		$start = microtime(true);

		/** @var \LiamW\PostMacros\Entity\Macro[] $macros */
		$finder = $this->finder('LiamW\PostMacros:Macro')
			->where('user_id', $this->userId)
			->order('macro_id');

		if ($lastOffset !== null)
		{
			$finder->where('macro_id', '>', $lastOffset);
		}

		$maxFetch = 100; // Macros can have a lot of dependencies, and therefore require a lot of memory for each entity, so don't take too many at a time.
		$macros = $finder->fetch($maxFetch);
		$fetchedMacros = count($macros);

		if (!$fetchedMacros)
		{
			return null; // done or nothing to do
		}

		foreach ($macros AS $macro)
		{
			$lastOffset = $macro->macro_id;

			$macro->delete();

			if ($maxRunTime && microtime(true) - $start > $maxRunTime)
			{
				return $lastOffset; // continue at this position
			}
		}

		if ($fetchedMacros == $maxFetch)
		{
			return $lastOffset; // more to do
		}
		else
		{
			return null;
		}
	}
}